@@ -33,6 +33,14 @@ from lvmcam.actor.actor import LvmcamActor
     type=str,
     help="Path to the user configuration file.",
 )
+@click.option(
+    "-r",
+    "--rmq_url",
+    "rmq_url",
+    default=None,
+    type=str,
+    help="rabbitmq url, eg: amqp://guest:guest@localhost:5672/",
+)
 @click.option(
     "-v",
     "--verbose",
@@ -46,10 +54,10 @@ from lvmcam.actor.actor import LvmcamActor
     help="Simulation mode. Overwrite configured camera type with skymakercam.",
 )
 @click.pass_context
-def lvmcam(ctx, config_file, verbose, simulate):
+def lvmcam(ctx, config_file, rmq_url, verbose, simulate):
     """lvm controller"""
 
-    ctx.obj = {"verbose": verbose, "config_file": config_file, "simulate": simulate}
+    ctx.obj = {"verbose": verbose, "config_file": config_file, "rmq_url": rmq_url, "simulate": simulate}
 
 
 @lvmcam.group(cls=DaemonGroup, prog="lvmcam_actor", workdir=os.getcwd())
@@ -58,11 +66,9 @@ def lvmcam(ctx, config_file, verbose, simulate):
 async def actor(ctx):
     """Runs the actor."""
 
-#    default_config_file = os.path.join(os.path.dirname(__file__), "etc/lvm.sci.agcam.yml")
-    config_file = ctx.obj["config_file"]
-    lvmcam_obj = LvmcamActor.from_config(config_file, verbose=ctx.obj["verbose"], simulate=ctx.obj["simulate"])
 
-    lvmcam_obj.log.debug("Hello world")
+    config_file = ctx.obj["config_file"]
+    lvmcam_obj = LvmcamActor.from_config(config_file, url=ctx.obj["rmq_url"], verbose=ctx.obj["verbose"], simulate=ctx.obj["simulate"])
 
     await lvmcam_obj.start()
     await lvmcam_obj.run_forever()
